package com.palserv.xml;

import java.util.*;

/**
XmlObject is the base (super) class of the various XML objects
in the com.palserv.xml package.
<p>
There are no public classes or methods in this file.
All are meant to be accessed through the
XmlGetFile and XmlVisitor interfaces.
*/
class XmlObject {
	protected ArrayList pis;
	protected String name;

/**
Accessor for the list of PIs (processing instructions)
*/
	public ArrayList getPis() { return pis; }

/**
Add a PI (processing instruction) to the list
*/
	public void addPi(PI p) {
		if (pis == null) {
			pis = new ArrayList();
		}
		pis.add(p);
	}

/**
Accessor for the name
*/
	public String getName() {
		return name;
	}

/**
"Set" method for the name
*/
	void setName(String n) {
		name = n;
	}

// polymorphic methods
// these should not be public
	void addChild(Entity child) {}
	void setContent(String c) {}
	void addAttribute(Attribute a) {}
	void accept(XmlVisitor x) {}
}

/**
Document.
Unlike the other subclasses of XmlObject, this does not
represent a part of an XML file, but rather, the whole
file.
*/
public class Document extends XmlObject {
	private Entity root;

	Entity getRoot() {
		return root;
	}

	void setRoot(Entity r) {
		root = r;
	}

	void accept(XmlVisitor x) {
		x.visitDocument(this);
	}
}

/**
Processing Instruction
*/
class PI extends XmlObject {
	private String value;


	String getValue() {
		return value;
	}

	PI(String n, String v) {
		name = n;
		value = v;
	}

	void accept(XmlVisitor x) {
		x.visitPI(this);
	}
}

/**
Entity
*/
class Entity extends XmlObject {
	private ArrayList children;
	private XmlObject parent;
	private ArrayList attributes;
	private boolean isRoot = false;
	private String tag;
	private String content;
	private boolean isEmpty;

	ArrayList getChildren() {
		return children;
	}

	XmlObject getParent() {
		return parent;
	}

	ArrayList getAttributes() {
		return attributes;
	}

	boolean getRoot() {
		return isRoot;
	}

	public String getName() {
		return getTag();
	}

	String getTag() {
		return tag;
	}

	String getContent() {
		return content;
	}

	boolean getEmpty() {
		return isEmpty;
	}

	void addChild(Entity child) {
		if (children == null) {
			children = new ArrayList();
		}
		children.add(child);
	}

	void setParent(XmlObject p) {
		parent = p;
	}

	void addAttribute(Attribute a) {
		if (attributes == null) {
			attributes = new ArrayList();
		}
		attributes.add(a);
	}

	void setRoot(boolean b) {
		isRoot = b;
	}

	void setTag(String t) {
		tag = t;
	}

	void setContent(String c) {
		content = c;
	}

	void setEmpty(boolean e) {
		isEmpty = e;
	}

	void accept(XmlVisitor x) {
		x.visitEntity(this);
	}
}

/**
Attribute
*/
class Attribute extends XmlObject {
	private String value;

	String getValue() {
		return value;
	}

	Attribute(String n, String v) {
		name = n;
		value = v;
	}

	void accept(XmlVisitor x) {
		x.visitAttribute(this);
	}
}
