package com.palserv.xml;

import java.io.*;
import java.util.*;

/**
This class should be used like this:
	XmlGetFile xgf = new XmlGetFile();
	xgf.getFile("sample.xml");  // file to process
After which you may want to do something like this:
	XmlPrintVisitor xpv = new XmlPrintVisitor();
	xpv.visitXml(xgf);
where XmlPrintVisitor extends XmlVisitor
*/
public class XmlGetFile {
	private boolean inTag;
	private boolean inComment;
	private boolean inSEPair;
	private boolean inPI;
	private HashSet tags;
	private ArrayList pairs;
	private Document doc;
	private XmlObject currObject;
	private ArrayList currObj;
	private int cnt = 0;
	private String tagText;
	private String content;
	private boolean isRoot;
	private String savTag;

	Document getDoc() { return doc; }

	void init() {
		doc = new Document();
		currObject = doc;
		currObj = new ArrayList();
		pairs = new ArrayList();
		tags = new HashSet();
		inTag = false;
		inComment = false;
		inSEPair = false;
		inPI = false;
		tagText = "";
		content = "";
		isRoot = true;
		savTag = "";
	}

/**
This is the public interface to the XmlGetFile class.
@param String fileName - the (fully qualified) file to process
*/
	public Document getFile(String fileName) {
		init();
		BufferedReader in = null;

		try {
			in = new BufferedReader(new FileReader(fileName));
		} catch(IOException e) {
			System.err.println("Unable to open " + fileName + ": " +
				e.getMessage());
		}

		currObj.add(doc);
		doc.setName(fileName);
		try {
			boolean flag = true;
			while (flag) {
				String iLine = in.readLine();
				cnt ++;
				if (iLine == null) {
					flag = false;
				} else {
					processLine(iLine);
				}
			}
			in.close();
		} catch(IOException e) {
			System.err.println("Unable to read from " + fileName + ": " +
				e.getMessage());
		}
		return doc;
	}

	void processLine(String line) {
		/* uses several instance variables:
		boolean inTag = false;
		boolean inComment = false;
		boolean inSEPair = false;
		boolean inPI = false;
		*/

		line = line.trim();

		if (line == null || line.length() == 0) {
			return;
		}

/*
These lines need some explanation.
First, we have eight declaration/assignments:
these establish the position of the first
occurrance of the critical strings.
*/
		int pLT = line.indexOf("<");
		int pLTS = line.indexOf("</");
		int pGT = line.indexOf(">");
		int pSGT = line.indexOf("/>");
		int pBegCom = line.indexOf("<!--");
		int pEndCom = line.indexOf("-->");
		int pLTQ = line.indexOf("<?");
		int pQGT = line.indexOf("?>");
/*
Then we invoke the getFirst method to find
the index and the value of the first occurance
of the four "start" strings (<, </, <?, <!--).
Then we do the same thing for the related "end"
strings (>, />, ?>, -->).

We use the SE class to return the results, which
contain an index and a value.

The index is the "index" 1-4 of the arguments passed.
The value is the actual value found.

Note that if two different strings begin at the same
offset (for example, "<" and "<?"), then getFirst will
return the higher index (later on the list of arguments).
*/
		SE ses = getFirst(pLT, pLTS, pLTQ, pBegCom);
		SE see = getFirst(pGT, pSGT, pQGT, pEndCom);
/*
Finally, we set four variables to the four index values
which can be returned from getFirst.
These are used in the code instead of hard-coded numbers.
*/
		int comNdx = 4;
		int piNdx = 3;
		int slshNdx = 2;
		int ltgtNdx = 1;

		if (inComment) {  // already in comment
			if (pEndCom != -1) {  // if it ends
				inComment = false;
				if (line.length() > (pEndCom + 3)) {
					String lin = line.substring(pEndCom + 3);
					processLine(lin);
				}
			}
			return;
		}

		if (ses.index == comNdx) {	// comment found?
			inComment = true;
			if (pBegCom > 1) {	// any thing before?
				String lin = line.substring(0, pBegCom - 1);
				processLine(lin);
			}
			return;
		}

		if (inPI) {  // in processing instruction?
			if (pQGT != -1) {  // end of pi found?
				inPI = false;
				if (line.length() > (pQGT + 2)) {
					String lin = line.substring(pQGT + 2);
					processLine(lin);
				}
			}
			return;
		}

		if (ses.index == piNdx) {	// processing instruction
			inPI = true;
			int ts = pLTQ + 2;

			int te = getEnd(line,ts);

			String name = line.substring(ts, te);
			if (see.index == piNdx & pLTQ < pQGT) {
				inPI = false;
				String val = line.substring(te + 1, pQGT);
				currObject.addPi(new PI(name, val));
				line = line.substring(pQGT + 2);
				processLine(line);
				return;
			} else {
				currObject.addPi(new PI(name, ""));
			}
			if (line.length() > te + 1) {
				line = line.substring(te + 1);
				processLine(line);
				return;
			}
		}

		if (inTag) {	// already within tag
			if (see.index == slshNdx) {  // end of empty tag?
				inTag = false;

				// capture tag text
				tagText += " " + line.substring(0,pSGT);
				addText();

				if (line.length() > (pSGT + 2)) {
					String lin = line.substring(pGT + 2);
					processLine(lin);
				}
				return;
			}

			if (see.index == ltgtNdx) {  // end of tag?
				// end of entity start tag
				inTag = false;
				inSEPair = true;

				// capture tag text
				tagText += " " + line.substring(0,pGT);
				addText();

				if (ses.index == ltgtNdx && pLT < pGT) {
					String tag = line.substring(pLT+1,pGT);
					pairs.add(tag);
				} else {
					pairs.add(savTag);
				}

				if (line.length() > (pGT + 1)) {
					String lin = line.substring(pGT + 1);
					processLine(lin);
				}
				return;
			}
			tagText += " " + line;
			return;
		}

		if (inSEPair) { 	// already within Start/End pair
			if (ses.index == slshNdx) {   // end of pair?
				content += line.substring(0, pLTS);
				currObject.setContent(content);
				content = "";
				int ts = pLTS + 2;
				int te = getEnd(line,ts);
				String tag = line.substring(ts,te);

				if (pairs.isEmpty()) {
					serr("end of pair, but nothing on pairs", line);
				}
				int ndx = pairs.size() - 1;
				if (!tag.equals((String)pairs.get(ndx))) {
					serr("end of pair, but no match; new tag: " + tag +
						"; pairs: " + (String)pairs.get(ndx), line);
				}

				pairs.remove(ndx);

				if (pairs.isEmpty()) {
					inSEPair = false;
				}

				int cndx = currObj.size() - 1;
				if (cndx < 0) {
					serr("curr index negative: " + cndx,line);
				}
				currObj.remove(cndx);
				if (cndx > 0) {
					currObject = (XmlObject)currObj.get(cndx - 1);
				} else {
					currObject = doc;
				}

				if (see.index == ltgtNdx) {
					if (te < pGT) {
						tagText += " " + line.substring(te, pGT);
						addText();
					}
					line = line.substring(pGT + 1);
					processLine(line);
				} else {
					if (line.length() > te + 1) {
						line = line.substring(te + 1);
						tagText += " " + line;
						processLine(line);
					}
				}
				return;
			}
		}

		if (ses.index == ltgtNdx) {    // tag found?
			if (pLT > 0) {
				content += line.substring(0,pLT);
			}
			int ts = pLT + 1;

			int te = getEnd(line,ts);

			String tag = line.substring(ts, te);
			addEnt(tag);

			inTag = true;
			if (see.index == ltgtNdx) {
				if (te < pGT) {
					tagText = line.substring(te, pGT);
					addText();
				}
				inTag = false;
				inSEPair = true;
				pairs.add(tag);
				if (line.length() > (pGT + 1)) {
					String lin = line.substring(pGT + 1);
					processLine(lin);
				}
				savTag = "";
				return;
			} else {
				savTag = tag;
			}
			if (line.length() > te + 1) {
				String lin = line.substring(te + 1);
				processLine(lin);
			}
			return;
		}

		// process content
		if (pLT > 1) {
			content += line.substring(0, pLT);
		}
	}

	void addText() {
		// string is in tagText

		// parse string into attributes (name="value")
		String tmp = tagText;
		boolean flag = true;
		while (flag) {
			tmp = tmp.trim();
			int pe = tmp.indexOf("=");
			if (pe == -1) {
				flag = false;
			} else {
				String nam = tmp.substring(0,pe);
				String dlm = tmp.substring(pe + 1, pe + 2);
				int pq = tmp.indexOf(dlm, pe + 2);
				if (pq == -1) {
					serr("Error in addText: no final delimiter: " + tmp, tagText);
				}
				String val = tmp.substring(pe + 2, pq);
				currObject.addAttribute(new Attribute(nam, val));
				tmp = tmp.substring(pq + 1);
			}
		}
		// reset accumulator
		tagText = "";
	}

	SE getFirst(int i1, int i2, int i3, int i4) {
		int l = 10000;
		int i = 10000;
		if (i1 <= l && i1 != -1) {
			l = i1;
			i = 1;
		}
		if (i2 <= l && i2 != -1) {
			l = i2;
			i = 2;
		}
		if (i3 <= l && i3 != -1) {
			l = i3;
			i = 3;
		}
		if (i4 <= l && i4 != -1) {
			l = i4;
			i = 4;
		}
		return new SE(l,i);
	}

	void addEnt(String tag) {
		tags.add(tag);
		Entity ent = new Entity();
		ent.setTag(tag);
		if (isRoot) {
			doc.setRoot(ent);
			isRoot = false;
			ent.setRoot(true);
			currObj.add(ent);
		} else {
			if (inSEPair) {
				if (inTag) {
					serr("addEnt: inSEPair and inTag; wha??","");
				} else {
					currObj.add(ent);
					currObject.addChild(ent);
					ent.setParent((XmlObject)currObject);
					ent.setRoot(false);
					ent.setEmpty(false);
				}
			} else {
				int cndx = currObj.size() - 2;
				if (cndx < 0) {
					serr("addEnt: not inSEPair but index negative","");
				}
				XmlObject x = (XmlObject)currObj.get(cndx);
				x.addChild(ent);
				ent.setParent((Entity)x);
				ent.setRoot(false);
				ent.setEmpty(true);
			}
		}
		currObject = ent;
	}

	int getEnd(String line, int ts) {
		int tes = line.indexOf(" ",ts);
		if (tes == -1) {
			tes = line.length();
		}
		int tesg = line.indexOf("/>",ts);
		if (tesg == -1) {
			tesg = line.length();
		}
		int teg = line.indexOf(">",ts);
		if (teg == -1) {
			teg = line.length();
		}
		int te = tes;
		if (te > tesg) {
			te = tesg;
		}
		if (te > teg) {
			te = teg;
		}

		if (ts < 0) {
			serr("index < 0 (ts): " + ts, line);
		}

		if (te < 0) {
			serr("index < 0 (te): " + te, line);
		}

		if (te < ts) {
			serr("te < ts: " + te + ", " + ts, line);
		}
		return te;
	}

	void serr(String m, String l) {
		System.err.println("Error in line " + cnt + ": " + m);
		System.err.println("Text of line ==>" + l + "<==");
		System.exit(0);
	}
}

class SE {
	public int lowest;
	public int index;
	SE(int l, int i) {
		lowest = l;
		index = i;
	}
}
