package com.palserv.xml;

import java.util.*;

/**
This class can be looked at as a sample of how to use
the XmlVisitor class.
The visitXxx methods (where Xxx is: PI, Document,
Entity, and Attribute) correspond to the Xxx classes
in XmlObject.  The default implementation
in XmlVisitor of these methods is to do nothing.
Thus if you wanted a visitor which ignored processing
instructions, don't override visitPI.
<p>
The findPIs method of XmlVisitor is invoked from three
of the four visitXxx methods.  It is not public and
probably should not be changed.  It gets the list
of PIs (processing instructions) and invokes
visitPI on each.
*/
public class XmlPrintVisitor extends XmlVisitor {

	void visitPI(PI p) {
		spt("  Processing Instruction name: " + p.getName());
		spt("    Processing Instruction value: " + p.getValue());
	}

	void visitDocument(Document d) {
		spt("Document: " + d.getName());
		findPIs(d);
	}

	void visitEntity(Entity e) {
		String ir = "";
		if (e.getRoot()) {
			ir = " is root";
		}
		spt("Entity: " + e.getName() + ir);
		if (!e.getRoot()) {
			spt("    Child of: " + e.getParent().getName());
		}
		String c = e.getContent();
		if (c != null && !c.equals("")) {
			spt("  Entity content: " + c);
		}
		findPIs(e);
		super.visitEntity(e);	// handles attributes and children
	}

	void visitAttribute(Attribute a) {
		spt("  Attribute Name: " + a.getName());
		spt("    Attribute Value: " + a.getValue());
		findPIs(a);
	}

/*
This is a "helper" method to print out a line.
It represents any "function" which is common to
more than one of the visitXxx methods.
*/
	void spt(String s) {
		System.out.println(s);
	}
}
