package com.palserv.xml;

import java.util.*;

public class XmlVisitor {
/* these methods should be overridden, if needed
*/
	void visitPI(PI p) {}
	void visitDocument(Document d) {}
	void visitEntity(Entity e) {
/* if you override this method (as you should), you
can say "super.visitEntity(e)" to get the functionality
here.  This code iterates to the attributes and children
*/
		ArrayList atts = e.getAttributes();
		if (atts != null) {
			Iterator i = atts.iterator();
			while (i.hasNext()) {
				Attribute a = (Attribute)i.next();
				a.accept(this);
			}
		}

		ArrayList kids = e.getChildren();
		if (kids != null) {
			Iterator i = kids.iterator();
			while (i.hasNext()) {
				Entity k = (Entity)i.next();
				k.accept(this);
			}
		}
	}
	void visitAttribute(Attribute a) {}

/* these methods probably should NOT be overridden
*/
	void findPIs(XmlObject x) {
		ArrayList p = x.getPis();
		if (p == null) {
			return;
		}
		Iterator i = p.iterator();
		while (i.hasNext()) {
			PI pi = (PI)i.next();
			visitPI(pi);
		}
	}

/**
This is the public interface to XmlVisitor.
<p>
In general, you can use this kind of code:
<blockquote>
	XmlGetFile xgf = new XmlGetFile();<br>
	Document doc = xgf.getFile("sample.xml");<br>
	XmlPrintVisitor xpv = new XmlPrintVisitor();<br>
	xpv.visitXml(doc);
</blockquote>
The first line builds an XmlGetFile object.
The second line invokes the getFile method.
The third line builds an XmlPrintVisitor object,
which extends XmlVisitor.
The fourth line invokes this method.
<p>
There should be no reason to override this method.
*/
	public void visitXml(Document doc) {
		doc.accept(this);
		Entity root = doc.getRoot();
		root.accept(this);
	}
}
